from odoo import tools, api, models, SUPERUSER_ID
from odoo.http import request
from odoo.tools import float_repr
import logging

_logger = logging.getLogger(__name__)


class Website(models.Model):
    _inherit = 'website'

    try:
        @api.model
        @tools.ormcache(
            'country_code', 'show_visible',
            'current_pl_id', 'website_pricelist_ids',
            'partner_pl_id', 'order_pl_id',
        )
        def _get_pl_partner_order(
            self, country_code, show_visible, current_pl_id, website_pricelist_ids,
            partner_pl_id=False, order_pl_id=False
        ):
            # Fake website_pricelist with partner_pricelist to avoid that a partner who use country_selector keeps the own pricelist
            # bypass: if not pricelists or partner.property_product_pricelist.id != website_pl:
            # use sudo to allow user from other company to access pricelist from website with company id = 1
            return super(Website, self.sudo())._get_pl_partner_order(country_code, show_visible, current_pl_id, website_pricelist_ids, partner_pl_id=False, order_pl_id=order_pl_id)
    except TypeError as e:
        # 8.0 compatbility for master.odoo.com
        if '__init__() takes at most 5' in str(e):
            pass
        else:
            raise

    def sale_product_domain(self):
        # remove apps products from the website content grid and list view
        # also ignore enterprise users
        um = self.env.ref('openerp_enterprise.product_user_month').id
        return ['&', '&'] + super(Website, self).sale_product_domain() + [('categ_id.name', '!=', 'Apps'), ('id', '!=', um)]

    # use 'assets' for minimal invalidations - ts used to recompute a new cached version on pl update
    @tools.ormcache('pl_id', 'ts', cache='assets')
    def _get_pricing_for_pricelist(self, pl_id, ts):
        def get_best_pricing(product, pl, recurrence="month"):
            return product._get_best_subscription_pricing_rule(
                **{'pricelist': pl, 'duration': 1, 'unit': recurrence}
            ).price
        pl = self.env['product.pricelist'].sudo().browse(pl_id)
        std = self.env.ref('openerp_enterprise.product_odoo_user_standard')
        std_disc = self.env.ref('openerp_enterprise.product_odoo_user_standard_discount')
        cst = self.env.ref('openerp_enterprise.product_odoo_user_custom')
        cst_disc = self.env.ref('openerp_enterprise.product_odoo_user_custom_discount')

        loc100 = self.env.ref('openerp_enterprise.product_maintenance')
        loc100_uom = self.env.ref('openerp_enterprise.product_uom_maintenance_100loc')

        prices = {
            'monthly_standard': get_best_pricing(std, pl),
            'monthly_standard_discount': get_best_pricing(std_disc, pl),
            'monthly_custom': get_best_pricing(cst, pl),
            'monthly_custom_discount': get_best_pricing(cst_disc, pl),
            'yearly_standard': get_best_pricing(std, pl, 'year') / 12.0,
            'yearly_standard_discount': get_best_pricing(std_disc, pl, 'year') / 12.0,
            'yearly_custom': get_best_pricing(cst, pl, 'year') / 12.0,
            'yearly_custom_discount': get_best_pricing(cst_disc, pl, 'year') / 12.0,
            'loc': pl._get_product_price(loc100, 1, uom=loc100_uom),
            'symbol': pl.currency_id.symbol.lstrip('US'),  # US = US$ -> we want $ |  MXN = Mxn$
            'pl': pl,
        }
        prices['monthly_standard_discounted'] = prices['monthly_standard'] + prices['monthly_standard_discount']
        prices['monthly_custom_discounted'] = prices['monthly_custom'] + prices['monthly_custom_discount']
        prices['yearly_standard_discounted'] = prices['yearly_standard'] + prices['yearly_standard_discount']
        prices['yearly_custom_discounted'] = prices['yearly_custom'] + prices['yearly_custom_discount']
        prices['pl_id'] = pl.id

        price_fields = [
            'yearly_standard_discounted', 'monthly_standard_discounted', 'yearly_standard', 'monthly_standard',
            'yearly_custom_discounted', 'monthly_custom_discounted', 'yearly_custom', 'monthly_custom'
        ]
        prices['show_decimal'] = any([float_repr(prices[field] % 1, 2) != '0.00' for field in price_fields])

        pack_starter = self.env.ref('openerp_enterprise.product_pack_partner_starter_product_template')
        pack_basic = self.env.ref('openerp_enterprise.product_pack_partner_basic_product_template')
        pack_standard = self.env.ref('openerp_enterprise.product_pack_partner_standard_product_template')
        pack_custom = self.env.ref('openerp_enterprise.product_pack_partner_custom_product_template')
        pack_pro = self.env.ref('openerp_enterprise.product_pack_partner_pro_product_template')

        prices['packs'] = {
            'discount': 15,
            'Starter': {
                'time': 4,
                'name': 'starter',
                'price': pack_starter._get_contextual_price(),
            },
            'Basic': {
                'time': 25,
                'name': 'basic',
                'price': pack_basic._get_contextual_price(),
            },
            'Standard': {
                'time': 50,
                'name': 'standard',
                'price': pack_standard._get_contextual_price(),
            },
            'Custom': {
                'time': 100,
                'name': 'custom',
                'price': pack_custom._get_contextual_price(),
            },
            'Pro': {
                'time': 200,
                'name': 'pro',
                'price': pack_pro._get_contextual_price(),
            },
        }

        odoo_sh_worker = self.env.ref('openerp_enterprise.product_odoo_sh_extra_worker')
        odoo_sh_storage = self.env.ref('openerp_enterprise.product_odoo_sh_extra_gb')
        odoo_sh_staging = self.env.ref('openerp_enterprise.product_odoo_sh_extra_staging')
        odoo_sh_dedicated = self.env.ref('openerp_enterprise.product_odoo_sh_dedicated')

        prices['odoo_sh'] = {
            'workers': get_best_pricing(odoo_sh_worker, pl),
            'storage': get_best_pricing(odoo_sh_storage, pl),
            'staging': get_best_pricing(odoo_sh_staging, pl),
            'dedicated': get_best_pricing(odoo_sh_dedicated, pl),
        }

        return prices

    @tools.ormcache('force_pl', 'country_code', cache='default')
    def get_cached_pricing_pl(self, country_code, force_pl=False):
        country_code = country_code  # cached - request.geoip.country_code
        return self.get_pricing_pl_id(force_pl=force_pl)

    def get_pricing_pl_id(self, force_pl=False):
        supported_pricelists = self.pricelist_ids.ids
        assert len(supported_pricelists) >= 1, 'No pricelists available for this website'
        pl = False

        # if pricelist forced by get param and valid
        force_pricelist = force_pl
        if force_pricelist and str(force_pricelist).isdigit():
            pl = self.env['product.pricelist'].browse(int(force_pricelist))

        # if code promo pricelist without recurring price or other exotic pl
        compatible = self.env.ref('openerp_enterprise.product_odoo_user_standard')._get_best_subscription_pricing_rule(
            **{'pricelist': pl, 'duration': 1, 'unit': 'month'}
        )
        if not compatible:
            pl = False
            request.session.pop('website_sale_current_pl', None)
            request.website.invalidate_recordset(['pricelist_id'])

        # else use current website pricelist (website sale mechanism)
        if not pl or not pl.exists():
            pl = request.website.pricelist_id

        # if pricelist not supported, fallback on first website pricelist (regardless country group !)
        if pl.id not in supported_pricelists:  # avoid gift coupon pricelist from ecommerce
            pl = self.env['product.pricelist'].browse(supported_pricelists[0])
        return pl.id

    def get_pricing(self, **kwargs):
        if self.is_public_user():
            country_code = request.geoip.country_code
            pl_id = self.get_cached_pricing_pl(country_code=country_code, force_pl=kwargs.get('pl'))
        else:
            pl_id = self.get_pricing_pl_id(force_pl=kwargs.get('pl'))
        pl = self.env['product.pricelist'].sudo().browse(pl_id)
        prices = self._get_pricing_for_pricelist(pl_id, pl.write_date.timestamp())
        prices['pl'] = pl
        prices['decimal_separator'] = request.lang.decimal_point
        return prices

    def odoo_get_order(self, periodicity, num_users=0, hosting='online', force_create=False, update_pricelist=False, **kwargs):
        sale_order = self.sale_get_order(force_create=force_create, update_pricelist=update_pricelist)

        plan = kwargs.get('plan', 'custom')
        assert plan in ('standard', 'custom'), "wrong plan"

        subscription_tmpl_ids = {
            'monthly': self.env.ref(f"openerp_enterprise.{plan}_template_odoo_monthly"),
            'yearly': self.env.ref(f"openerp_enterprise.{plan}_template_odoo_yearly"),
            'yearly_learning': self.env.ref('openerp_enterprise.quotation_template_odoo_partnership'),
        }

        odoo_learning_product_id = self.env.ref('openerp_enterprise.odoo_learning_platform_access').sudo()

        # we remove all the order line that are related to
        sol_to_unlink = self.env['sale.order.line']
        for line in sale_order.order_line:
            if line.product_id.is_odoo_app or line.product_id.is_odoo_user or line.product_id.id == odoo_learning_product_id.id:
                sol_to_unlink |= line
        sol_to_unlink.sudo().unlink()

        if periodicity == 'yearly_learning':  # if we are creating an Odoo learning partnership cart
            sale_order._cart_update(product_id=odoo_learning_product_id.id, set_qty=1)
        else:
            # # the IoT app depends only on the amount of IoT Boxes set on the pricing page.
            # if 'iot' in apps:
            #     apps.remove('iot')

            #     num_iot_boxes = int(kwargs['num_iot_boxes'])
            #     if num_iot_boxes > 0:
            #         sale_order._cart_update(product_id=self.env.ref('openerp_enterprise.product_iot_box').sudo().id, set_qty=num_iot_boxes)

            # Add the users / users discount into the order line
            sale_order._cart_update(product_id=self.env.ref(f'openerp_enterprise.product_odoo_user_{plan}').id, set_qty=num_users)
            sale_order._cart_update(product_id=self.env.ref(f'openerp_enterprise.product_odoo_user_{plan}_discount').id, set_qty=num_users)

            # Add the product Odoo.sh
            if hosting == 'odoo_sh':
                sale_order._cart_update(product_id=self.env.ref('openerp_enterprise.product_odoo_sh_extra_worker').sudo().id, set_qty=max(int(kwargs['odoosh_workers']), 1))
                sale_order._cart_update(product_id=self.env.ref('openerp_enterprise.product_odoo_sh_extra_gb').sudo().id, set_qty=max(int(kwargs['odoosh_storage']), 1))
                sale_order._cart_update(product_id=self.env.ref('openerp_enterprise.product_odoo_sh_extra_staging').sudo().id, set_qty=max(int(kwargs['odoosh_staging']), 1))
                if int(kwargs.get('odoosh_dedicated', 0)):
                    sale_order._cart_update(product_id=self.env.ref('openerp_enterprise.product_odoo_sh_dedicated').sudo().id, set_qty=1)

            if kwargs.get('maintenance_code') == "1":
                # cart_update remove line if qty set to 0, so we add line with qty 1, and reset it after manually
                line_info = sale_order._cart_update(product_id=self.env.ref('openerp_enterprise.product_maintenance').sudo().id, set_qty=1)
                sale_order.order_line.browse(line_info['line_id']).product_uom_qty = 0

        quote_tmpl_id = subscription_tmpl_ids.get(periodicity, subscription_tmpl_ids['yearly']).sudo()
        # since the 'quotation_template_odoo_partnership' is linked to the partnership quote template, we have
        # to manually set the quote template to learning partnership
        if periodicity == 'yearly_learning':
            quote_tmpl_id = self.env.ref('openerp_enterprise.quotation_template_odoo_learning_partnership').sudo()

        values = {
            'sale_order_template_id': quote_tmpl_id.id,
            'referrer_id': None,
            'client_order_ref': kwargs.get('db_uuid'),
            'user_id': False,
        }
        sub_hosting = {'online': 'saas', 'odoo_sh': 'paas', 'on_premise': 'premise'}.get(hosting)
        if sub_hosting:
            values['hosting'] = sub_hosting

        # check if the user selected an integrating partner
        if kwargs.get('implementationService') == 'partner':
            integrating_partner_id = kwargs.get('integrating_partner_id') and int(kwargs['integrating_partner_id'])
            if integrating_partner_id:
                integrating_partner = self.env['res.partner'].sudo().browse(integrating_partner_id)
                values['referrer_id'] = integrating_partner_id
                values['user_id'] = integrating_partner.exists().user_id.id or SUPERUSER_ID

        # Compute which tag should be set on the SO, by default OE Self tag is set
        tag_ids = [self.env.ref('openerp_enterprise.self_subscription_crm_tag').id]
        # check if the user wants a success pack
        if kwargs.get('implementationService') == 'success_pack':
            success_pack_tag_id = self.env.ref('openerp_enterprise.success_pack_{}_tag'.format(kwargs.get("pack")), raise_if_not_found=False)
            if success_pack_tag_id:
                tag_ids.append(success_pack_tag_id.id)
        # Set the needed tags
        values['tag_ids'] = [(4, tag_id, 0) for tag_id in tag_ids]

        # force team id to false if not saleman - avoid _get_default_team_id
        if not values.get('user_id'):
            values['team_id'] = False

        # write the values on the sale order
        sale_order.write(values)

        return sale_order

    def sale_get_order(self, *args, **kwargs):
        so = super().sale_get_order(*args, **kwargs)
        if so.pricelist_id.sale_company_id and so.pricelist_id.sale_company_id != so.company_id:
            # reset company to match pricelist, but re-force pricelist too to avoid automatic recompute!
            so.write({'company_id': so.pricelist_id.sale_company_id.id, 'pricelist_id': so.pricelist_id.id})
        return so
